<?
/*
Copyright (c) 2009, 2010, Andreas 'Segaja' Schleifer

Permission to use, copy, modify, and/or distribute this software for any purpose with or
without fee is hereby granted, provided that the above copyright notice and this permission
notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
OR PERFORMANCE OF THIS SOFTWARE.
*/


class Object { // Object Class, von der Channel und User abstammen
  protected $data = array(); // Array zum speichern der Attribute

  public function __get($key) { // magic getter method; liefert Element $key aus dem Attribut-Array
    return $this->data[$key];
  }
}

final class Channel extends Object { // Class Channel, die von Object abstammt
  public function __construct() { // Constructor, Arrays leer initiieren
    $this->data['children'] = array();
    $this->data['user'] = array();
  }

  public function __set($key, $value) { // magic setter method; setzt Element $key des Attribut-Array auf $value
    if($key == 'children' || $key == 'user') { // wenn $key "children" oder "user"...
      if(is_array($value)) { // wenn $value ein Array ist...
        $this->data[$key] = $value; // $value in Attribut-Array bei $key speichern
      }
      else { // wenn $value kein Array ist...
        $type = ($key == 'children') ? 'channel' : $key; // $type festlegen (ein child ist ein channel)

        // wenn $value ein Object vom Typ $type ist...
        if(is_object($value) && strtolower(get_class($value)) == $type) {
          $this->data[$key][$value->id] = $value; // $value in Attribut-Array richtig speichern
        }
        else { // Exception werfen...
          throw new Exception("If \$key is '$key', \$value must be an object.");
        }
      }
    }
    else { // sonst...
      $this->data[$key] = $value; // $value an Stelle $key in Attribut-Array speichern
    }
  }

  public function getData() { // Gibt alles ausser die Children und die User zurueck...
    $tmp = $this->data;

    unset($tmp['children'], $tmp['user']);

    return $tmp;
  }

  public function getArrayData($key) { // Gibt alle Daten des Elements $key zurueck
    return $this->data[$key];
  }
}

final class User extends Object { // Class User, die von Object abstammt
  public function __set($key, $value) { // magic setter method; setzt Element $key des Attribut-Array auf $value
    $this->data[$key] = $value;
  }

  public function getData() { // Gibt alle Daten zurueck
    return $this->data;
  }
}
?>
